/***********************************************************************
 c_demo.c - Simple demo for CPS USB high voltage probes.

 Jim Lamberson
 CPS

 Copyright 2015, All Rights Reserved.

 c_demo.c is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation, either version 3 of the License, or
 (at your option) any later version.

 c_demo.c is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with c_demo.c. If not, see <http://www.gnu.org/licenses/>.

************************************************************************/

#include <stdio.h>
#include <ctype.h>

#include "HVProbe.h"

static void MenuLoop(void);
static int OpenAllProbes(void);

static int nprobes;             // number of probes detected
static int activeprobe = 0;     // id of active probe

///////////////////////////////////////////////////
// Top-level function

int main()
{
    printf("CPS HV PROBE DEMO\n\n");

    nprobes = HVP_OpenSystem();     // open the probe api
    if (nprobes < 0)                // if error opening api
        printf("HVP_OpenSystem() error: %s\n", HVP_ErrorString(nprobes));
    else {
        printf("%d probe%s detected%s\n", nprobes, (nprobes == 1) ? "" : "s", nprobes ? ":" : "");
        if (nprobes > 0) {          // if at least one probe detected
            if (OpenAllProbes() == HVP_ERR_OK)  // open and display attributes of all detected probes
                MenuLoop();                     // accept and process user commands
        }
        HVP_CloseSystem();  // close all probes and close the api
    }
    return 0;   // terminate app
}

////////////////////////////////////////////////////////////
// Open and display attributes of all detected probes

static int OpenAllProbes(void)
{
    int probeid;
    PROBE_INFO info;
    int errcode = HVP_ERR_OK;

    for (probeid = 0; probeid < nprobes; probeid++) // for each detected probe ...
    {
        printf("probe%d - ", probeid);

        errcode = HVP_OpenProbe(probeid);   // open the probe
        if (errcode == HVP_ERR_CHECKSUM)
            printf("UNCALIBRATED! ");
        else if (errcode != HVP_ERR_OK) {
            printf("HVP_OpenProbe() error: %s\n", HVP_ErrorString(errcode));
            break;
        }

        errcode = HVP_GetAttributes(probeid, &info);   // get probe attributes
        if (errcode) {
            printf("HVP_GetAttributes() error: %s\n", HVP_ErrorString(errcode));
            break;
        }

        printf("model %d, ", info.model);   // display attributes
        printf("SN %d, ", info.serialnum);
        printf("fw =%s\n", info.firmware_info);
    }
    return errcode;
}

//////////////////////////////////////
// Display user command menu

static void ShowMenu(void)
{
    printf("\nCommands:\n");
    printf("v - read DC Volts\n");
    printf("s - probe status\n");
    if (nprobes > 1)                                    // if multiple probes detected
        printf("0-%d - select probe\n", nprobes - 1);   //   allow user to select active probe
    printf("m - command menu\n");
    printf("q - quit demo\n\n");
}

///////////////////////////////////
// Prompt user for command

static void prompt(void)
{
    printf("probe%d>", activeprobe);
}

///////////////////////////////////////////////////
// Wait for and return a keyboard char

#ifdef __linux__
#include <termios.h>
#include <unistd.h>
int getch(void)
{
    struct termios oldattr, newattr;
    int ch;
    tcgetattr( STDIN_FILENO, &oldattr );
    newattr = oldattr;
    newattr.c_lflag &= ~( ICANON | ECHO );
    tcsetattr( STDIN_FILENO, TCSANOW, &newattr );
    ch = getchar();
    tcsetattr( STDIN_FILENO, TCSANOW, &oldattr );
    return ch;
}
#elif _WIN32
#include <conio.h>
#define getch() _getch()
#endif

/////////////////////////////////////////////////////////////////////
// Accept and process user commands until Quit command received

static void MenuLoop(void)
{
    int errcode;
    int status;
    double buf[NUM_METERS];

    ShowMenu();
    prompt();

    while (1)
    {
        char c = getch();
        switch (tolower(c))
        {
        case 'v':
            printf("v\n");
            errcode = HVP_GetMeters(activeprobe, buf);   // read meter data from probe
            if (errcode)
                printf("HVP_GetMeters() error: %s\n", HVP_ErrorString(errcode));
            else
                printf("%f VDC\n", buf[METER_DC]);
            break;

        case 's':
            printf("s\n");
            errcode = HVP_GetStatus(activeprobe, &status);   // get probe status
            if (errcode)
                printf("HVP_GetStatus() error: %s\n", HVP_ErrorString(errcode));
            else if (status & HVP_STATUS_ALARM)
                printf("status=WARNING: Probe is near HV with ground lead unconnected!\n");
            else
                printf("status=OK\n");
            break;

        case 'm':   // refresh menu
            printf("m\n");
            ShowMenu();
            break;

        case 'q':   // quit
            return;

        default:
            if (nprobes > 1) {
                if (isdigit(c)) {
                    int probeid = c - 0x30;
                    if (probeid < nprobes) {
                        printf("%c\n", c);
                        activeprobe = probeid;  // select active probe
                    }
                }
            }
            continue;
        }

        prompt();
    }
}
